"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.importCheck = exports.default = void 0;

var path = _interopRequireWildcard(require("path"));

var _utils = require("./utils");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const isStylesDirFile = (appDirectory, currentFilePath, importPath) => {
  if (!currentFilePath) {
    return false;
  }

  const importFileAbsPath = path.join(path.dirname(currentFilePath), importPath);
  const stylesAbsPath = path.resolve(appDirectory, 'styles');
  return importFileAbsPath.includes(stylesAbsPath);
};

const checkIsOutsideSrc = (filename, importName, srcDir) => {
  if (!filename || filename.includes('node_modules/.block-tools/source')) {
    return false;
  }

  const currentFileDir = path.dirname(filename);
  const importFileAbsPath = path.resolve(currentFileDir, importName);
  return path.relative(srcDir, importFileAbsPath).includes('..');
};

const importCheck = () => ({
  name: 'import-check',
  visitor: {
    ImportDeclaration({
      node
    }, {
      opts,
      file
    }) {
      const {
        source
      } = node;
      const {
        appDirectory
      } = opts;
      const {
        filename
      } = file.opts;
      const srcDir = `${appDirectory}/src`;
      const importName = source !== null && source !== void 0 && source.value ? source.value : undefined;

      if (!importName) {
        return;
      }

      if (!(0, _utils.isProjectFile)(importName)) {
        return;
      }

      if (isStylesDirFile(appDirectory, filename, importName)) {
        throw new Error(`Importing files in 'styles' directory is not allowed: '${importName}', You can place the file in the 'src' directory or remove the imported code`);
      }

      if (checkIsOutsideSrc(filename, importName, srcDir)) {
        throw new Error(`Importing files outside of 'src' directory is not allowed: '${importName}, You can place the file in the 'src' directory and modify the imported code'`);
      }
    }

  }
});

exports.importCheck = importCheck;
var _default = importCheck;
exports.default = _default;